using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Timer=System.Windows.Forms.Timer;

namespace ThreadingReports
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }
        private LongRunningReport rpt = null;
        private Thread reportThread = null;
        private int recordCount = 0;
        private Timer timer;

        private void Form1_Load(object sender, EventArgs e)
        {
            this.Show();
            this.StartNewReport();
        }

        private void btnStopReport_Click(object sender, EventArgs e)
        {
            timer1.Enabled = false;
            label1.Visible = false;
            this.StopReport();
        }

        private void btnStartNewReport_Click(object sender, EventArgs e)
        {
            this.StartNewReport();
        }

        private void StopReport()
        {
            try
            {
                if (rpt == null)
                    return;
                StopTimer();
                recordCount = 0;
                rpt.Stop();

                if (this.reportThread != null)
                {
                    if (this.reportThread.IsAlive)
                    {
                        this.reportThread.Abort();
                    }
                }
            }
            catch (Exception)
            {
                System.Diagnostics.Debug.WriteLine("An error occured while stopping the report.");
            }
        }

        private void StartNewReport()
        {
            timer1.Enabled = false;
            label1.Visible = true;
            label1.Text = "";

            this.StopReport();

            // Initialize the report
            this.rpt = new LongRunningReport();

            //The next few lines are just to provide a dummy datasource for the report
            //A datatable with lots of records to make it a large report.
            DataTable dt = new DataTable();
            dt.Columns.Add("Index", typeof(int));

            for (int i = 1; i < 40000; i++)
            {
                dt.Rows.Add(i);
            }
            rpt.DataSource = dt;


            //Make an event handler for FetchData
            rpt.FetchData += new GrapeCity.ActiveReports.SectionReport.FetchEventHandler(rpt_FetchData);

            // Set the total number of records to the Maximum.
            progressBar1.Minimum = 0;
            progressBar1.Maximum = rpt.GetNoOfRecords(rpt.DataSource);

            //Now, create a timer which will check the progress periodically.
            timer = new Timer();
            timer.Interval = 100;
            timer.Tick += new EventHandler(CheckProgress);
            timer.Start();

            // Hook it up to the document
            this.viewer1.Document = rpt.Document;
            // Create a thread to execute the Run method in:
            this.reportThread = new Thread(new ThreadStart(rpt.Run));
            // Start the thread:
            this.reportThread.Start();
        }

        void rpt_FetchData(object sender, GrapeCity.ActiveReports.SectionReport.FetchEventArgs eArgs)
        {
            recordCount++;
        }

   
        protected override void Dispose(bool disposing)
        {
            // Make sure that report runner thread is not still running, or the process won't stop.
            this.StopReport();

            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        private void CheckProgress(object sender, EventArgs e)
        {
            if (progressBar1.Maximum > recordCount)
            {
                progressBar1.Value = recordCount;
                label1.Text = ((int)((progressBar1.Value * 100) / progressBar1.Maximum)).ToString() + "%";
            }
            else
            {
                timer1.Enabled = true;
                label1.Text = "Done!";
                progressBar1.Value = progressBar1.Maximum;
                StopTimer();
            }
        }

        private void StopTimer()
        {
            recordCount = 0;
            timer.Stop();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            //This timer is used to show that the report generation is over
            timer1.Interval = 500;
            label1.Visible = !label1.Visible;
        }
    }
}